﻿using System;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Collections.Generic;

namespace TextFileUtil
{
    public class ConvertTextFile
    {


        /// <summary>
        /// 
        /// </summary>
        /// <param name="args"></param>
        /// <returns>変換に成功した場合は、変換後のファイルのパス。失敗した場合はnull。</returns>
        public static string Convert( string[] args )
        {
            List<string> argsList = new List<string>( args.Length );
            for( int i = 0; i < args.Length; i++ )
            {
                if( !string.IsNullOrEmpty( args[i] ) )
                {
                    args[i] = args[i].Trim();
                    if( !string.IsNullOrEmpty( args[i] ) )
                    {
                        argsList.Add( args[i] );
                    }
                }
            }

            Convert( argsList.ToArray(), true );

            string output = null;
            if( sucseeded )
            {
                output = outputFilePath;
            }
            else
            {
            }

            return output;
        }


        /// <summary>
        /// メッセージを出力する。
        /// </summary>
        /// <param name="text"></param>
        private static void WriteLineMessage( string text, ErrorMessageMode mode )
        {
            if( mode == ErrorMessageMode.MsgBox )
            {
                MessageBox.Show( text );
            }
            else
            {
                Console.WriteLine( text );
            }
        }

        private enum ErrorMessageMode
        {
            Console,
            MsgBox,
            ThrowException
        }

        private static void ErrorMessage( string message, ErrorMessageMode mode, Exception exception )
        {
            switch( mode )
            {
                case ErrorMessageMode.Console:
                    Console.WriteLine( "エラー : " + message );
                    break;
                case ErrorMessageMode.MsgBox:
                    MessageBox.Show( "エラー : " + message );
                    break;
                case ErrorMessageMode.ThrowException:
                default:
                    throw exception;
            }
        }


        #region private static string GetOption( string[] args, string optionName )

        /// <summary>
        /// <para>引数から、指定されたオプション名に続く値を取得する。</para>
        /// <para>取得できなかった場合はnullを返す。</para>
        /// </summary>
        /// <param name="args">引数が格納された文字列の配列</param>
        /// <param name="optionName">取得するオプションの名前</param>
        /// <returns>指定されたオプション名に続く値</returns>
        private static string GetOption( string[] args, string optionName )
        {
            if( args == null ) { throw new ArgumentNullException( "args" ); }
            if( string.IsNullOrEmpty( optionName ) ) { throw new ArgumentException( "Argumet is null or empty.", "optionName" ); }

            string option = null;
            int end = args.Length - 1;
            for( int i = 0; i < end; i++ )
            {
                if( args[i] == optionName )
                {
                    if( i + 1 < args.Length )
                    {
                        option = args[i + 1];
                        break;
                    }
                }
            }

            return option;
        }

        #endregion


        #region private static bool IsOptionDefined( string[] args, string optionName )

        /// <summary>
        /// <para>指定されたオプションが渡されているかどうかを取得する。</para>
        /// </summary>
        /// <param name="args"></param>
        /// <param name="optionName"></param>
        /// <returns></returns>
        private static bool IsOptionDefined( string[] args, string optionName )
        {
            if( args == null ) { throw new ArgumentNullException( "args" ); }
            if( string.IsNullOrEmpty( optionName ) ) { throw new ArgumentException( "Argumet is null or empty.", "optionName" ); }

            bool defined = false;
            int end = args.Length;
            for( int i = 0; i < end; i++ )
            {
                if( args[i] == optionName )
                {
                    defined = true;
                    break;
                }
            }

            return defined;
        }

        #endregion


        #region private static Encoding ParseEncoding( string name )

        /// <summary>
        /// <para>文字列のエンコーディングの名前から、Encodingオブジェクトを作成する。</para>
        /// <para>無効な名前などが渡された場合はnullを返す。</para>
        /// <para>
        /// 指定可能な値 : 
        /// default(システムのデフォルト),
        /// ASCII, UTF-7, UTF-8, UTF-16, UTF-32,
        /// IANA登録名(Shift_JIS, EUC-JP, など),
        /// string.Emptyまたはnull(例外をスローせずにnullを返す)
        /// </para>
        /// </summary>
        /// <param name="name">エンコーディングの名前</param>
        /// <returns>Encodingオブジェクト</returns>
        private static Encoding ParseEncoding( string name )
        {
            if( string.IsNullOrEmpty( name ) ) { return null; }

            Encoding enc = null;
            string lower = name.ToLower();
            if( lower == "def" || lower == "default" )
            {
                enc = Encoding.Default;
            }
            else if( lower == "ascii" )
            {
                enc = Encoding.ASCII;
            }
            else if( lower == "utf-7" )
            {
                enc = Encoding.UTF7;
            }
            else if( lower == "utf-8" )
            {
                enc = Encoding.UTF8;
            }
            else if( lower == "utf-16" )
            {
                enc = Encoding.Unicode;
            }
            else if( lower == "utf-32" )
            {
                enc = Encoding.UTF32;
            }
            else
            {
                enc = System.Text.Encoding.GetEncoding( name );
            }

            return enc;
        }

        #endregion


        #region private const string ManText (操作説明のテキスト)

        /// <summary>
        /// 操作説明のテキスト。
        /// </summary>
        private const string ManText =
    @"変換するテキストファイルは
最初の引数にファイルのパスを渡すか、
もしくは、 -f [ファイルパス] でファイルを指定します。
また、必要に応じて以下のオプションが指定可能です。
  -mb : メッセージをメッセージボックスで表示します。
  -n : 改行コードを ""\n"" に変換します。
  -rn : 改行コードを ""\r\n"" に変換します。
  -ie [encode] : 変換元のテキストファイルのエンコードを指定します。
  -oe [encode] : 変換後のテキストファイルのエンコードを指定します。
    [encode] には、ASCII, UTF-7, UTF-8, UTF-16, UTF-32,
    default(OSのデフォルトのエンコード)
    または、IANA登録名を指定できます。
    -ie または -oe を指定しない場合は
    実行環境でのデフォルトのエンコードが使用されます。
  man : このメッセージを表示します。";

        #endregion


        // Main() ////////////////////////////////////////////


        static void Main( string[] args )
        {
            Convert( args, false );
        }


        //  ////////////////////////////////////////////


        private static void Convert( string[] args, bool throwException )
        {
            sucseeded = false;
            resultMessage = null;
            outputFilePath = null;

            ErrorMessageMode errMsgMode;
            if( throwException )
            {
                errMsgMode = ErrorMessageMode.ThrowException;
            }
            else
            {
                errMsgMode = (IsOptionDefined( args, "-mb" ) ? ErrorMessageMode.MsgBox : ErrorMessageMode.Console);
            }


            #region 操作説明を出力( "man" オプション )
            if( args.Length == 0 || IsOptionDefined( args, "man" ) )
            {
                if( errMsgMode != ErrorMessageMode.ThrowException )
                {
                    WriteLineMessage( ManText, errMsgMode );
                }

                return;
            }
            #endregion


            #region 入力ファイルのパスを取得

            string filePath = GetOption( args, "-f" );
            if( string.IsNullOrEmpty( filePath ) )
            {
                filePath = args[0];
            }
            if( !File.Exists( filePath ) )
            {
                string msg = "ファイルパスが正しく指定されていません。";
                ErrorMessage( msg, errMsgMode, new ArgumentException( msg ) );

                return;
            }

            #endregion


            #region 保存ファイル名を決定する。

            string newFilePath = null;
            {
                string fileExt = Path.GetExtension( filePath );
                string fileName = string.IsNullOrEmpty( fileExt ) ? filePath : filePath.Substring( 0, filePath.LastIndexOf( fileExt ) );
                string addition = "_";
                newFilePath = fileName + addition + fileExt;

                while( File.Exists( newFilePath ) )
                {
                    addition += "_";
                    newFilePath = fileName + addition + fileExt;
                }
            }

            #endregion


            //入力ファイルのエンコード
            Encoding srcEnc = Encoding.Default;

            //出力ファイルのエンコード
            Encoding outEnc = null;


            #region 入力ファイルのエンコード指定

            if( args.Length >= 2 && IsOptionDefined( args, "-ie" ) )
            {
                string srcEncStr = null;
                try
                {
                    //入力ファイルのエンコード指定を取得
                    srcEncStr = GetOption( args, "-ie" );
                    if( string.IsNullOrEmpty( srcEncStr ) )
                    {
                        throw new ArgumentException( "入力のエンコードを指定するオプション -ie が渡されましたが、値がありません。" );
                    }
                    else
                    {
                        Encoding tmp = ParseEncoding( srcEncStr );
                        if( tmp != null )
                        {
                            srcEnc = tmp;
                        }
                        else
                        {
                            throw new ArgumentException( "入力に指定されたエンコードの名前 \"" + srcEncStr + "\" は正しくありません。" );
                        }
                    }
                }
                catch( Exception ex )
                {
                    if( errMsgMode == ErrorMessageMode.ThrowException )
                    {
                        throw ex;
                    }
                    else
                    {
                        ErrorMessage( ex.Message, errMsgMode, null );
                    }

                    return;
                }
            }

            #endregion


            #region 出力ファイルのエンコード指定

            // 指定無しの場合、 outEnc は null のまま
            //   -> 後の処理で、srcEncと同じエンコードが設定される。

            if( args.Length >= 2 && IsOptionDefined( args, "-oe" ) )
            {
                try
                {
                    string outEncStr = GetOption( args, "-oe" );
                    if( string.IsNullOrEmpty( outEncStr ) )
                    {
                        throw new ArgumentException( "出力のエンコードを指定するオプション -oe が渡されましたが、値がありません。" );
                    }
                    else
                    {
                        Encoding tmp = ParseEncoding( outEncStr );
                        if( tmp != null )
                        {
                            outEnc = tmp;
                        }
                        else
                        {
                            throw new ArgumentException( "出力に指定されたエンコードの名前 \"" + outEncStr + "\" は正しくありません。" );
                        }
                    }
                }
                catch( Exception ex )
                {
                    if( errMsgMode == ErrorMessageMode.ThrowException )
                    {
                        throw ex;
                    }
                    else
                    {
                        ErrorMessage( ex.Message, errMsgMode, null );
                    }

                    return;
                }
            }

            #endregion


            #region 改行コードの指定

            string newLineCode = Environment.NewLine;
            {
                bool n = IsOptionDefined( args, "-n" );
                bool rn = IsOptionDefined( args, "-rn" );
                if( n && rn )
                {
                    string errMessage = "-n と -rn の両方を指定することはできません。";
                    ErrorMessage( errMessage, errMsgMode, new ArgumentException( errMessage ) );

                    return;
                }

                if( n )
                {
                    newLineCode = "\n";
                }
                else if( rn )
                {
                    newLineCode = "\r\n";
                }
            }

            #endregion


            #region 変換の実行

            try
            {
                Convert( filePath, newFilePath, srcEnc, outEnc, newLineCode );
                sucseeded = true;
                outputFilePath = newFilePath;
            }
            catch( Exception ex )
            {
                if( errMsgMode == ErrorMessageMode.ThrowException )
                {
                    throw ex;
                }
                else
                {
                    string msg = ex.Message;
                    if( ex.InnerException != null )
                    {
                        msg += " : " + ex.InnerException.Message;
                    }
                    ErrorMessage( msg, errMsgMode, ex );
                }
            }
            finally
            {
            }

            #endregion



        }//Convert(...)


        private static bool sucseeded = false;
        private static string outputFilePath = null;
        private static string resultMessage = null;



        #region public static void ConvertTextFile( ... )

        /// <summary>
        /// テキストファイルのエンコードと改行コードを変換する。
        /// </summary>
        /// <param name="filePath">変換するファイルのパス。</param>
        /// <param name="newFilePath">変換後のテキストを保存するファイルのパス。</param>
        /// <param name="srcEnc">入力ファイルのエンコード。</param>
        /// <param name="outEnc">変換後のエンコード。
        /// nullを渡すとエンコードは変更されない。</param>
        /// <param name="newLineCode">変換後の改行コード。</param>
        public static void Convert( string filePath, string newFilePath, Encoding srcEnc, Encoding outEnc, string newLineCode )
        {
            //入力ファイルから読み取った文字列
            string srcText = null;

            #region 入力ファイルの読み込み
            {
                StreamReader sr = null;
                try
                {
                    sr = new StreamReader( filePath, srcEnc );
                    srcText = sr.ReadToEnd();
                    if( outEnc == null )
                    {
                        outEnc = sr.CurrentEncoding;
                    }
                }
                catch( Exception ex )
                {
                    throw new Exception( "ファイルを読み取れませんでした。", ex );
                }
                finally
                {
                    if( sr != null )
                    {
                        sr.Close();
                    }
                }
            }
            #endregion


            #region 出力ファイルへの書き込み
            {
                StreamWriter sw = null;
                try
                {
                    sw = new StreamWriter( newFilePath, false, outEnc );
                    StringReader strReader = new StringReader( srcText );
                    string line;
                    while( (line = strReader.ReadLine()) != null )
                    {
                        sw.Write( line + newLineCode );
                    }
                }
                catch( Exception ex )
                {
                    throw new Exception( "変換後のテキストを書き込めませんでした。", ex );
                }
                finally
                {
                    if( sw != null )
                    {
                        sw.Close();
                    }
                }
            }
            #endregion
        }

        #endregion

    }
}