﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

using TextFileUtil;

namespace ConvertTextFileGUI
{
    public partial class Form1 : Form
    {

        public Form1()
        {
            InitializeComponent();

            bool filePathPassed = false;
            if( Program.Args.Length > 0 )
            {
                string path = Program.Args[0];
                if( File.Exists( path ) )
                {
                    filePathPassed = true;
                    this.tbFileName.Text = path;
                    this.openFileDialog.InitialDirectory = Path.GetDirectoryName( path );
                }
            }
            
            if( !filePathPassed )
            {
                this.openFileDialog.InitialDirectory = System.Environment.GetFolderPath( Environment.SpecialFolder.Desktop );
            }

            Encoding defEnc = Encoding.Default;
            this.lbDefEncName.Text = defEnc.EncodingName;
            //string defEncName = defEnc.EncodingName + "(system default)";
            //this.cbSourceEnc.Items[0] = defEncName;
            //this.cbOUtputEnc.Items[0] = defEncName;

            this.cbSourceEnc.Items.Clear();
            this.cbOutputEnc.Items.Clear();

            EncInfoItem noChange = new EncInfoItem( null, "No Change" );
            this.cbOutputEnc.Items.Add( noChange );

            //EncInfoItem defEncItem = new EncInfoItem( defEnc, "System Default");
            //this.AddEncInfoItem( defEncItem );

            EncInfoItem ascii = new EncInfoItem( Encoding.ASCII, "ASCII" );
            this.AddEncInfoItem( ascii );

            EncInfoItem sjis = new EncInfoItem( Encoding.GetEncoding( "Shift_JIS" ), "Shift JIS" );
            this.AddEncInfoItem( sjis );

            EncInfoItem jis = new EncInfoItem( Encoding.GetEncoding( "ISO-2022-JP" ), "JIS (ISO-2022-JP)" );
            this.AddEncInfoItem( jis );

            EncInfoItem euc = new EncInfoItem( Encoding.GetEncoding( "EUC-JP" ), "EUC-JP" );
            this.AddEncInfoItem( euc );

            EncInfoItem mac = new EncInfoItem( Encoding.GetEncoding( "x-Mac-Japanese" ), "MacJapanese (x-Mac-Japanese)" );
            this.AddEncInfoItem( mac );

            EncInfoItem u7 = new EncInfoItem( Encoding.UTF7, "UTF-7" );
            this.AddEncInfoItem( u7 );

            EncInfoItem u8 = new EncInfoItem( Encoding.UTF8, "UTF-8" );
            this.AddEncInfoItem( u8 );

            EncInfoItem u16 = new EncInfoItem( Encoding.Unicode, "UTF-16" );
            this.AddEncInfoItem( u16 );

            EncInfoItem utf16b = new EncInfoItem( Encoding.BigEndianUnicode, "UTF-16 (big endian)" );
            this.AddEncInfoItem( utf16b );

            EncInfoItem u32 = new EncInfoItem( Encoding.UTF32, "UTF-32" );
            this.AddEncInfoItem( u32 );

            this.cbSourceEnc.SelectedIndex = 0;
            this.cbOutputEnc.SelectedIndex = 0;

            //EncodingInfo[] encs = Encoding.GetEncodings();
            //EncInfoItem[] items = new EncInfoItem[encs.Length];
            //for( int i = 0; i < encs.Length; i++ )
            //{
            //    items[i] = new EncInfoItem( encs[i] );
            //}

            //Array.Sort(
            //    items,
            //    delegate( EncInfoItem a, EncInfoItem b )
            //    {
            //        //return a.Name.CompareTo( b.Name );
            //        return a.CodePage - b.CodePage;
            //    }
            //    );

            //foreach( EncInfoItem item in items )
            //{
            //    this.cbSourceEnc.Items.Add( item );
            //    this.cbOutputEnc.Items.Add( item );
            //}

            if( Environment.NewLine == "\r\n" )
            {
                this.cbNewLineCode.SelectedIndex = 0;
            }
            else
            {
                this.cbNewLineCode.SelectedIndex = 1;
            }
        }

        private void AddEncInfoItem( EncInfoItem enc )
        {
            this.cbSourceEnc.Items.Add( enc );
            this.cbOutputEnc.Items.Add( enc );
        }


        private void btnSelectFile_Click( object sender, EventArgs e )
        {
            if( this.openFileDialog.ShowDialog() == DialogResult.OK )
            {
                this.tbFileName.Text = this.openFileDialog.FileName;
            }
        }

        #region テキストボックスへのファイルドロップ

        private void tbFileName_DragEnter( object sender, DragEventArgs e )
        {
            if( e.Data.GetDataPresent( DataFormats.FileDrop ) )
            {
                e.Effect = DragDropEffects.Link;//とりあえずリンクで。
            }
            else
            {
                e.Effect = DragDropEffects.None;
                return;
            }
        }

        private void tbFileName_DragDrop( object sender, DragEventArgs e )
        {
            if( !e.Data.GetDataPresent( DataFormats.FileDrop ) )
            {
                return;
            }

            string[] pathList = (string[])e.Data.GetData( DataFormats.FileDrop );

            if( pathList.Length == 0 ) { return; }

            this.tbFileName.Text = pathList[0];
        }

        #endregion


        private void btnConvert_Click( object sender, EventArgs e )
        {
            string path = this.tbFileName.Text;
            if( string.IsNullOrEmpty( path ) )
            {
                SetStatusMessage( "変換するファイルが指定されていません。" );
                return;
            }
            if( !File.Exists( path ) )
            {
                SetStatusMessage( "指定されたファイルが見つかりません。" );
                return;
            }

            string ext = Path.GetExtension( path ).ToLower();
            if( ext == ".exe" )
            {
                SetStatusMessage( ".exeファイルを変換することはできません。" );
                return;
            }
            else if( ext == ".dll" )
            {
                SetStatusMessage( ".dllファイルを変換することはできません。");
                return;
            }

            List<string> args = new List<string>();
            args.Add( "-f" );
            args.Add( path );
            args.Add( "-mb" );

            {
                string encName;
                if( this.cbSourceEnc.SelectedIndex < 0 )
                {
                    encName = this.cbSourceEnc.Text;
                }
                else
                {
                    EncInfoItem enc = this.cbSourceEnc.SelectedItem as EncInfoItem;
                    encName = enc.IanaName;
                }

                args.Add( "-ie" );
                args.Add( encName );
            }

            {
                string encName;
                bool noChange = false;

                if( this.cbOutputEnc.SelectedIndex < 0 )
                {
                    encName = this.cbOutputEnc.Text;
                }
                //else if( this.cbOutputEnc.SelectedIndex == 1 )
                //{
                //    encName = "default";
                //}
                else
                {
                    EncInfoItem enc = this.cbOutputEnc.SelectedItem as EncInfoItem;
                    if( enc.IsEmpty )
                    {
                        noChange = true;
                        encName = "";
                    }
                    else
                    {
                        encName = enc.IanaName;
                    }
                }

                if( !noChange )
                {
                    args.Add( "-oe" );
                    args.Add( encName );
                }
            }

            if( this.cbNewLineCode.SelectedIndex != 0 )
            {
                args.Add( "-n" );
            }

            SetStatusMessage( "変換中 ..." );

            string outputFilePath = null;
            try
            {
                outputFilePath = ConvertTextFile.Convert( args.ToArray() );
            }
            catch( Exception ex )
            {
                SetStatusMessage( ex.Message );
                return;
            }


            SetStatusMessage( "完了 > " + Path.GetFileName( outputFilePath ) );

            //if( this.cbOpenOutputFile.Checked )
            //{
            //    System.Diagnostics.Process.Start( "notepad.exe " + outputFilePath );
            //}
        }


        /// <summary>
        /// ステータスバーにメッセージを表示する。
        /// </summary>
        /// <param name="message"></param>
        private void SetStatusMessage( string message )
        {
            this.statusLabel.Text = message;
            this.statusStrip1.Refresh();
        }

    }
}
